#version 3.1;

#switch (playerClock)
  #case (0)
    #declare helmetColor    = color rgb <1,0,0>;
    #declare faceColor      = color rgb <0,1,0>;
    #declare bodyColor      = color rgb <0,0,1>;
    #declare handsFeetColor = color rgb <0,0,0>;
    #declare armsLegsColor  = color rgb <0,0,0>;
    #declare backpackColor  = color rgb <0,0,0>;

    #declare floorColor = color rgb <1/3,1/3,1/3>;
  #break

  #case (1)
    #declare helmetColor    = color rgb <0,0,0>;
    #declare faceColor      = color rgb <0,0,0>;
    #declare bodyColor      = color rgb <0,0,0>;
    #declare handsFeetColor = color rgb <1,0,0>;
    #declare armsLegsColor  = color rgb <0,1,0>;
    #declare backpackColor  = color rgb <0,0,1>;

    #declare floorColor = color rgb <1/3,1/3,1/3>;
  #break

  #case (2)
    #declare helmetColor    = color rgb <0,0,0>;
    #declare faceColor      = color rgb <0,0,0>;
    #declare bodyColor      = color rgb <0,0,0>;
    #declare handsFeetColor = color rgb <0,0,0>;
    #declare armsLegsColor  = color rgb <0,0,0>;
    #declare backpackColor  = color rgb <0,0,0>;

    #declare floorColor = color rgb <0,1/3,1>;
  #break

#end

/*
Old version with 4 pictures

#version 3.1

#switch (playerClock)
  #case (0)
    #declare helmetColor    = color rgb <1,0,0>;
    #declare faceColor      = color rgb <0,1,0>;
    #declare bodyColor      = color rgb <0,0,0>;
    #declare handsFeetColor = color rgb <0,0,0>;
    #declare armsLegsColor  = color rgb <0,0,0>;
    #declare backpackColor  = color rgb <0,0,0>;

    #declare floorColor = color rgb <1/3,1/3,1/3>;
  #break

  #case (1)
    #declare helmetColor    = color rgb <0,0,0>;
    #declare faceColor      = color rgb <0,0,0>;
    #declare bodyColor      = color rgb <1,0,0>;
    #declare handsFeetColor = color rgb <0,1,0>;
    #declare armsLegsColor  = color rgb <0,0,0>;
    #declare backpackColor  = color rgb <0,0,0>;

    #declare floorColor = color rgb <1/3,1/3,1/3>;
  #break

  #case (2)
    #declare helmetColor    = color rgb <0,0,0>;
    #declare faceColor      = color rgb <0,0,0>;
    #declare bodyColor      = color rgb <0,0,0>;
    #declare handsFeetColor = color rgb <0,0,0>;
    #declare armsLegsColor  = color rgb <1,0,0>;
    #declare backpackColor  = color rgb <0,1,0>;

    #declare floorColor = color rgb <1/3,1/3,1/3>;
  #break

  #case (3)
    #declare helmetColor    = color rgb <0,0,0>;
    #declare faceColor      = color rgb <0,0,0>;
    #declare bodyColor      = color rgb <0,0,0>;
    #declare handsFeetColor = color rgb <0,0,0>;
    #declare armsLegsColor  = color rgb <0,0,0>;
    #declare backpackColor  = color rgb <0,0,0>;

    #declare floorColor = color rgb <1,1,1>;
  #break

#end
*/
